/* TimedQueue.c generated by valac 0.12.0, the Vala compiler
 * generated from TimedQueue.vala, do not modify */

/* Copyright 2010-2011 Yorba Foundation
 *
 * This software is licensed under the GNU Lesser General Public License
 * (version 2.1 or later).  See the COPYING file in this distribution. 
 */
/* TimedQueue is a specialized collection class.  It holds items in order, but rather than being*/
/* manually dequeued, they are dequeued automatically after a specified amount of time has elapsed*/
/* for that item.  As of today, it's possible the item will be dequeued a bit later than asked*/
/* for, but it will never be early.  Future implementations might tighten up the lateness.*/
/**/
/* The original design was to use a signal to notify when an item has been dequeued, but Vala has*/
/* a bug with passing an unnamed type as a signal parameter:*/
/* https://bugzilla.gnome.org/show_bug.cgi?id=628639*/
/**/
/* The rate the items come off the queue can be spaced out.  Note that this can cause items to back*/
/* up.  As of today, TimedQueue makes no effort to combat this.*/

#include <glib.h>
#include <glib-object.h>
#include <gee.h>
#include <gobject/gvaluecollector.h>


#define TYPE_TIMED_QUEUE (timed_queue_get_type ())
#define TIMED_QUEUE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_TIMED_QUEUE, TimedQueue))
#define TIMED_QUEUE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_TIMED_QUEUE, TimedQueueClass))
#define IS_TIMED_QUEUE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_TIMED_QUEUE))
#define IS_TIMED_QUEUE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_TIMED_QUEUE))
#define TIMED_QUEUE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_TIMED_QUEUE, TimedQueueClass))

typedef struct _TimedQueue TimedQueue;
typedef struct _TimedQueueClass TimedQueueClass;
typedef struct _TimedQueuePrivate TimedQueuePrivate;

#define TYPE_SORTED_LIST (sorted_list_get_type ())
#define SORTED_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SORTED_LIST, SortedList))
#define SORTED_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SORTED_LIST, SortedListClass))
#define IS_SORTED_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SORTED_LIST))
#define IS_SORTED_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SORTED_LIST))
#define SORTED_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SORTED_LIST, SortedListClass))

typedef struct _SortedList SortedList;
typedef struct _SortedListClass SortedListClass;

#define TIMED_QUEUE_TYPE_ELEMENT (timed_queue_element_get_type ())
#define TIMED_QUEUE_ELEMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TIMED_QUEUE_TYPE_ELEMENT, TimedQueueElement))
#define TIMED_QUEUE_ELEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TIMED_QUEUE_TYPE_ELEMENT, TimedQueueElementClass))
#define TIMED_QUEUE_IS_ELEMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TIMED_QUEUE_TYPE_ELEMENT))
#define TIMED_QUEUE_IS_ELEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TIMED_QUEUE_TYPE_ELEMENT))
#define TIMED_QUEUE_ELEMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TIMED_QUEUE_TYPE_ELEMENT, TimedQueueElementClass))

typedef struct _TimedQueueElement TimedQueueElement;
typedef struct _TimedQueueElementClass TimedQueueElementClass;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
typedef struct _TimedQueueElementPrivate TimedQueueElementPrivate;
#define _timed_queue_element_unref0(var) ((var == NULL) ? NULL : (var = (timed_queue_element_unref (var), NULL)))
#define _g_destroy_func0(var) (((var == NULL) || (g_destroy_func == NULL)) ? NULL : (var = (g_destroy_func (var), NULL)))
typedef struct _TimedQueueParamSpecElement TimedQueueParamSpecElement;
typedef struct _ParamSpecTimedQueue ParamSpecTimedQueue;

#define TYPE_HASH_TIMED_QUEUE (hash_timed_queue_get_type ())
#define HASH_TIMED_QUEUE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_HASH_TIMED_QUEUE, HashTimedQueue))
#define HASH_TIMED_QUEUE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_HASH_TIMED_QUEUE, HashTimedQueueClass))
#define IS_HASH_TIMED_QUEUE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_HASH_TIMED_QUEUE))
#define IS_HASH_TIMED_QUEUE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_HASH_TIMED_QUEUE))
#define HASH_TIMED_QUEUE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_HASH_TIMED_QUEUE, HashTimedQueueClass))

typedef struct _HashTimedQueue HashTimedQueue;
typedef struct _HashTimedQueueClass HashTimedQueueClass;
typedef struct _HashTimedQueuePrivate HashTimedQueuePrivate;

typedef void (*DequeuedCallback) (gconstpointer item, void* user_data);
struct _TimedQueue {
	GTypeInstance parent_instance;
	volatile int ref_count;
	TimedQueuePrivate * priv;
};

struct _TimedQueueClass {
	GTypeClass parent_class;
	void (*finalize) (TimedQueue *self);
	void (*notify_dequeued) (TimedQueue* self, gconstpointer item);
	void (*clear) (TimedQueue* self);
	gboolean (*contains) (TimedQueue* self, gconstpointer item);
	gboolean (*enqueue) (TimedQueue* self, gconstpointer item);
	gboolean (*enqueue_many) (TimedQueue* self, GeeCollection* items);
	gboolean (*remove_first) (TimedQueue* self, gconstpointer item);
	void (*paused) (TimedQueue* self, gboolean is_paused);
	gint (*get_size) (TimedQueue* self);
};

struct _TimedQueuePrivate {
	GType g_type;
	GBoxedCopyFunc g_dup_func;
	GDestroyNotify g_destroy_func;
	guint hold_msec;
	DequeuedCallback callback;
	gpointer callback_target;
	GDestroyNotify callback_target_destroy_notify;
	GEqualFunc equal_func;
	gint priority;
	guint timer_id;
	SortedList* queue;
	guint dequeue_spacing_msec;
	gulong last_dequeue;
	gboolean paused_state;
};

typedef gint64 (*Comparator) (void* a, void* b, void* user_data);
struct _TimedQueueElement {
	GTypeInstance parent_instance;
	volatile int ref_count;
	TimedQueueElementPrivate * priv;
	gpointer item;
	gulong ready;
};

struct _TimedQueueElementClass {
	GTypeClass parent_class;
	void (*finalize) (TimedQueueElement *self);
};

struct _TimedQueueElementPrivate {
	GType g_type;
	GBoxedCopyFunc g_dup_func;
	GDestroyNotify g_destroy_func;
};

struct _TimedQueueParamSpecElement {
	GParamSpec parent_instance;
};

struct _ParamSpecTimedQueue {
	GParamSpec parent_instance;
};

struct _HashTimedQueue {
	TimedQueue parent_instance;
	HashTimedQueuePrivate * priv;
};

struct _HashTimedQueueClass {
	TimedQueueClass parent_class;
};

struct _HashTimedQueuePrivate {
	GType g_type;
	GBoxedCopyFunc g_dup_func;
	GDestroyNotify g_destroy_func;
	GeeHashMap* item_count;
};


static gpointer timed_queue_parent_class = NULL;
static gpointer timed_queue_element_parent_class = NULL;
static gpointer hash_timed_queue_parent_class = NULL;

gpointer timed_queue_ref (gpointer instance);
void timed_queue_unref (gpointer instance);
GParamSpec* param_spec_timed_queue (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_timed_queue (GValue* value, gpointer v_object);
void value_take_timed_queue (GValue* value, gpointer v_object);
gpointer value_get_timed_queue (const GValue* value);
GType timed_queue_get_type (void) G_GNUC_CONST;
GType sorted_list_get_type (void) G_GNUC_CONST;
static gpointer timed_queue_element_ref (gpointer instance);
static void timed_queue_element_unref (gpointer instance);
static GParamSpec* timed_queue_param_spec_element (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) G_GNUC_UNUSED;
static void timed_queue_value_set_element (GValue* value, gpointer v_object) G_GNUC_UNUSED;
static void timed_queue_value_take_element (GValue* value, gpointer v_object) G_GNUC_UNUSED;
static gpointer timed_queue_value_get_element (const GValue* value) G_GNUC_UNUSED;
static GType timed_queue_element_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
#define TIMED_QUEUE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_TIMED_QUEUE, TimedQueuePrivate))
enum  {
	TIMED_QUEUE_DUMMY_PROPERTY
};
TimedQueue* timed_queue_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, guint hold_msec, DequeuedCallback callback, void* callback_target, GEqualFunc equal_func, gint priority);
TimedQueue* timed_queue_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, guint hold_msec, DequeuedCallback callback, void* callback_target, GEqualFunc equal_func, gint priority);
static gint64 timed_queue_element_comparator (void* a, void* b);
static gint64 _timed_queue_element_comparator_comparator (void* a, void* b, gpointer self);
SortedList* sorted_list_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, Comparator cmp, void* cmp_target);
SortedList* sorted_list_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, Comparator cmp, void* cmp_target);
static guint timed_queue_get_heartbeat_timeout (TimedQueue* self);
static gboolean timed_queue_on_heartbeat (TimedQueue* self);
static gboolean _timed_queue_on_heartbeat_gsource_func (gpointer self);
guint timed_queue_get_dequeue_spacing_msec (TimedQueue* self);
void timed_queue_set_dequeue_spacing_msec (TimedQueue* self, guint msec);
void timed_queue_notify_dequeued (TimedQueue* self, gconstpointer item);
static void timed_queue_real_notify_dequeued (TimedQueue* self, gconstpointer item);
gboolean timed_queue_is_paused (TimedQueue* self);
void timed_queue_pause (TimedQueue* self);
void timed_queue_unpause (TimedQueue* self);
void timed_queue_clear (TimedQueue* self);
static void timed_queue_real_clear (TimedQueue* self);
gboolean timed_queue_contains (TimedQueue* self, gconstpointer item);
static gboolean timed_queue_real_contains (TimedQueue* self, gconstpointer item);
gboolean timed_queue_enqueue (TimedQueue* self, gconstpointer item);
static gboolean timed_queue_real_enqueue (TimedQueue* self, gconstpointer item);
static gulong timed_queue_calc_ready_time (TimedQueue* self);
static TimedQueueElement* timed_queue_element_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, gconstpointer item, gulong ready);
static TimedQueueElement* timed_queue_element_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, gconstpointer item, gulong ready);
gboolean timed_queue_enqueue_many (TimedQueue* self, GeeCollection* items);
static gboolean timed_queue_real_enqueue_many (TimedQueue* self, GeeCollection* items);
gboolean sorted_list_add_list (SortedList* self, GeeList* items);
gboolean timed_queue_remove_first (TimedQueue* self, gconstpointer item);
static gboolean timed_queue_real_remove_first (TimedQueue* self, gconstpointer item);
gulong now_ms (void);
gpointer sorted_list_get_at (SortedList* self, gint index);
gpointer sorted_list_remove_at (SortedList* self, gint index);
gint timed_queue_get_size (TimedQueue* self);
void timed_queue_paused (TimedQueue* self, gboolean is_paused);
static void timed_queue_real_paused (TimedQueue* self, gboolean is_paused);
#define TIMED_QUEUE_ELEMENT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TIMED_QUEUE_TYPE_ELEMENT, TimedQueueElementPrivate))
enum  {
	TIMED_QUEUE_ELEMENT_DUMMY_PROPERTY
};
static void timed_queue_element_finalize (TimedQueueElement* obj);
static void timed_queue_finalize (TimedQueue* obj);
GType hash_timed_queue_get_type (void) G_GNUC_CONST;
#define HASH_TIMED_QUEUE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_HASH_TIMED_QUEUE, HashTimedQueuePrivate))
enum  {
	HASH_TIMED_QUEUE_DUMMY_PROPERTY
};
HashTimedQueue* hash_timed_queue_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, guint hold_msec, DequeuedCallback callback, void* callback_target, GHashFunc hash_func, GEqualFunc equal_func, gint priority);
HashTimedQueue* hash_timed_queue_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, guint hold_msec, DequeuedCallback callback, void* callback_target, GHashFunc hash_func, GEqualFunc equal_func, gint priority);
static void hash_timed_queue_real_notify_dequeued (TimedQueue* base, gconstpointer item);
static void hash_timed_queue_removed (HashTimedQueue* self, gconstpointer item);
static void hash_timed_queue_real_clear (TimedQueue* base);
static gboolean hash_timed_queue_real_contains (TimedQueue* base, gconstpointer item);
static gboolean hash_timed_queue_real_enqueue (TimedQueue* base, gconstpointer item);
static gboolean hash_timed_queue_real_enqueue_many (TimedQueue* base, GeeCollection* items);
static gboolean hash_timed_queue_real_remove_first (TimedQueue* base, gconstpointer item);
static void hash_timed_queue_finalize (TimedQueue* obj);


static gint64 _timed_queue_element_comparator_comparator (void* a, void* b, gpointer self) {
	gint64 result;
	result = timed_queue_element_comparator (a, b);
	return result;
}


static gboolean _timed_queue_on_heartbeat_gsource_func (gpointer self) {
	gboolean result;
	result = timed_queue_on_heartbeat (self);
	return result;
}


TimedQueue* timed_queue_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, guint hold_msec, DequeuedCallback callback, void* callback_target, GEqualFunc equal_func, gint priority) {
	TimedQueue* self = NULL;
	DequeuedCallback _tmp0_;
	GEqualFunc _tmp1_ = NULL;
	SortedList* _tmp3_ = NULL;
	guint _tmp4_;
	guint _tmp5_;
	self = (TimedQueue*) g_type_create_instance (object_type);
	self->priv->g_type = g_type;
	self->priv->g_dup_func = g_dup_func;
	self->priv->g_destroy_func = g_destroy_func;
	self->priv->hold_msec = hold_msec;
	_tmp0_ = callback;
	(self->priv->callback_target_destroy_notify == NULL) ? NULL : (self->priv->callback_target_destroy_notify (self->priv->callback_target), NULL);
	self->priv->callback = NULL;
	self->priv->callback_target = NULL;
	self->priv->callback_target_destroy_notify = NULL;
	self->priv->callback = _tmp0_;
	self->priv->callback_target = callback_target;
	self->priv->callback_target_destroy_notify = NULL;
	if (equal_func != NULL) {
		_tmp1_ = equal_func;
	} else {
		GEqualFunc _tmp2_ = NULL;
		_tmp2_ = gee_functions_get_equal_func_for (g_type);
		_tmp1_ = _tmp2_;
	}
	self->priv->equal_func = _tmp1_;
	self->priv->priority = priority;
	_tmp3_ = sorted_list_new (TIMED_QUEUE_TYPE_ELEMENT, (GBoxedCopyFunc) timed_queue_element_ref, timed_queue_element_unref, _timed_queue_element_comparator_comparator, NULL);
	_g_object_unref0 (self->priv->queue);
	self->priv->queue = _tmp3_;
	_tmp4_ = timed_queue_get_heartbeat_timeout (self);
	_tmp5_ = g_timeout_add_full (priority, _tmp4_, _timed_queue_on_heartbeat_gsource_func, timed_queue_ref (self), timed_queue_unref);
	self->priv->timer_id = _tmp5_;
	return self;
}


TimedQueue* timed_queue_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, guint hold_msec, DequeuedCallback callback, void* callback_target, GEqualFunc equal_func, gint priority) {
	return timed_queue_construct (TYPE_TIMED_QUEUE, g_type, g_dup_func, g_destroy_func, hold_msec, callback, callback_target, equal_func, priority);
}


guint timed_queue_get_dequeue_spacing_msec (TimedQueue* self) {
	guint result = 0U;
	g_return_val_if_fail (IS_TIMED_QUEUE (self), 0U);
	result = self->priv->dequeue_spacing_msec;
	return result;
}


void timed_queue_set_dequeue_spacing_msec (TimedQueue* self, guint msec) {
	guint _tmp0_;
	guint _tmp1_;
	g_return_if_fail (IS_TIMED_QUEUE (self));
	if (msec == self->priv->dequeue_spacing_msec) {
		return;
	}
	if (self->priv->timer_id != 0) {
		g_source_remove (self->priv->timer_id);
	}
	self->priv->dequeue_spacing_msec = msec;
	_tmp0_ = timed_queue_get_heartbeat_timeout (self);
	_tmp1_ = g_timeout_add_full (self->priv->priority, _tmp0_, _timed_queue_on_heartbeat_gsource_func, timed_queue_ref (self), timed_queue_unref);
	self->priv->timer_id = _tmp1_;
}


static guint timed_queue_get_heartbeat_timeout (TimedQueue* self) {
	guint result = 0U;
	guint _tmp0_ = 0U;
	guint _tmp1_;
	g_return_val_if_fail (IS_TIMED_QUEUE (self), 0U);
	if (self->priv->dequeue_spacing_msec == 0) {
		_tmp0_ = self->priv->hold_msec / 10;
	} else {
		_tmp0_ = self->priv->dequeue_spacing_msec / 2;
	}
	_tmp1_ = CLAMP (_tmp0_, (guint) 10, G_MAXUINT);
	result = _tmp1_;
	return result;
}


static void timed_queue_real_notify_dequeued (TimedQueue* self, gconstpointer item) {
	g_return_if_fail (IS_TIMED_QUEUE (self));
	self->priv->callback (item, self->priv->callback_target);
}


void timed_queue_notify_dequeued (TimedQueue* self, gconstpointer item) {
	TIMED_QUEUE_GET_CLASS (self)->notify_dequeued (self, item);
}


gboolean timed_queue_is_paused (TimedQueue* self) {
	gboolean result = FALSE;
	g_return_val_if_fail (IS_TIMED_QUEUE (self), FALSE);
	result = self->priv->paused_state;
	return result;
}


void timed_queue_pause (TimedQueue* self) {
	g_return_if_fail (IS_TIMED_QUEUE (self));
	if (self->priv->paused_state) {
		return;
	}
	self->priv->paused_state = TRUE;
	g_signal_emit_by_name (self, "paused", TRUE);
}


void timed_queue_unpause (TimedQueue* self) {
	g_return_if_fail (IS_TIMED_QUEUE (self));
	if (!self->priv->paused_state) {
		return;
	}
	self->priv->paused_state = FALSE;
	g_signal_emit_by_name (self, "paused", FALSE);
}


static void timed_queue_real_clear (TimedQueue* self) {
	g_return_if_fail (IS_TIMED_QUEUE (self));
	gee_collection_clear (GEE_COLLECTION (self->priv->queue));
}


void timed_queue_clear (TimedQueue* self) {
	TIMED_QUEUE_GET_CLASS (self)->clear (self);
}


static gboolean timed_queue_real_contains (TimedQueue* self, gconstpointer item) {
	gboolean result = FALSE;
	g_return_val_if_fail (IS_TIMED_QUEUE (self), FALSE);
	{
		GeeIterator* _tmp0_ = NULL;
		GeeIterator* _e_it;
		_tmp0_ = gee_iterable_iterator (GEE_ITERABLE (self->priv->queue));
		_e_it = _tmp0_;
		while (TRUE) {
			gboolean _tmp1_;
			gpointer _tmp2_ = NULL;
			TimedQueueElement* e;
			gboolean _tmp3_;
			_tmp1_ = gee_iterator_next (_e_it);
			if (!_tmp1_) {
				break;
			}
			_tmp2_ = gee_iterator_get (_e_it);
			e = (TimedQueueElement*) _tmp2_;
			_tmp3_ = self->priv->equal_func (item, e->item);
			if (_tmp3_) {
				result = TRUE;
				_timed_queue_element_unref0 (e);
				_g_object_unref0 (_e_it);
				return result;
			}
			_timed_queue_element_unref0 (e);
		}
		_g_object_unref0 (_e_it);
	}
	result = FALSE;
	return result;
}


gboolean timed_queue_contains (TimedQueue* self, gconstpointer item) {
	return TIMED_QUEUE_GET_CLASS (self)->contains (self, item);
}


static gboolean timed_queue_real_enqueue (TimedQueue* self, gconstpointer item) {
	gboolean result = FALSE;
	gulong _tmp0_;
	TimedQueueElement* _tmp1_ = NULL;
	TimedQueueElement* _tmp2_;
	gboolean _tmp3_;
	gboolean _tmp4_;
	g_return_val_if_fail (IS_TIMED_QUEUE (self), FALSE);
	_tmp0_ = timed_queue_calc_ready_time (self);
	_tmp1_ = timed_queue_element_new (self->priv->g_type, (GBoxedCopyFunc) self->priv->g_dup_func, self->priv->g_destroy_func, item, _tmp0_);
	_tmp2_ = _tmp1_;
	_tmp3_ = gee_collection_add (GEE_COLLECTION (self->priv->queue), _tmp2_);
	_tmp4_ = _tmp3_;
	_timed_queue_element_unref0 (_tmp2_);
	result = _tmp4_;
	return result;
}


gboolean timed_queue_enqueue (TimedQueue* self, gconstpointer item) {
	return TIMED_QUEUE_GET_CLASS (self)->enqueue (self, item);
}


static gboolean timed_queue_real_enqueue_many (TimedQueue* self, GeeCollection* items) {
	gboolean result = FALSE;
	gulong _tmp0_;
	gulong ready_time;
	GeeArrayList* _tmp1_ = NULL;
	GeeArrayList* elements;
	gboolean _tmp7_;
	g_return_val_if_fail (IS_TIMED_QUEUE (self), FALSE);
	g_return_val_if_fail (GEE_IS_COLLECTION (items), FALSE);
	_tmp0_ = timed_queue_calc_ready_time (self);
	ready_time = _tmp0_;
	_tmp1_ = gee_array_list_new (TIMED_QUEUE_TYPE_ELEMENT, (GBoxedCopyFunc) timed_queue_element_ref, timed_queue_element_unref, NULL);
	elements = _tmp1_;
	{
		GeeIterator* _tmp2_ = NULL;
		GeeIterator* _item_it;
		_tmp2_ = gee_iterable_iterator (GEE_ITERABLE (items));
		_item_it = _tmp2_;
		while (TRUE) {
			gboolean _tmp3_;
			gpointer _tmp4_ = NULL;
			gpointer item;
			TimedQueueElement* _tmp5_ = NULL;
			TimedQueueElement* _tmp6_;
			_tmp3_ = gee_iterator_next (_item_it);
			if (!_tmp3_) {
				break;
			}
			_tmp4_ = gee_iterator_get (_item_it);
			item = _tmp4_;
			_tmp5_ = timed_queue_element_new (self->priv->g_type, (GBoxedCopyFunc) self->priv->g_dup_func, self->priv->g_destroy_func, item, ready_time);
			_tmp6_ = _tmp5_;
			gee_abstract_collection_add (GEE_ABSTRACT_COLLECTION (elements), _tmp6_);
			_timed_queue_element_unref0 (_tmp6_);
			((item == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (item = (self->priv->g_destroy_func (item), NULL));
		}
		_g_object_unref0 (_item_it);
	}
	_tmp7_ = sorted_list_add_list (self->priv->queue, GEE_LIST (elements));
	result = _tmp7_;
	_g_object_unref0 (elements);
	return result;
}


gboolean timed_queue_enqueue_many (TimedQueue* self, GeeCollection* items) {
	return TIMED_QUEUE_GET_CLASS (self)->enqueue_many (self, items);
}


static gboolean timed_queue_real_remove_first (TimedQueue* self, gconstpointer item) {
	gboolean result = FALSE;
	GeeIterator* _tmp0_ = NULL;
	GeeIterator* iter;
	g_return_val_if_fail (IS_TIMED_QUEUE (self), FALSE);
	_tmp0_ = gee_iterable_iterator (GEE_ITERABLE (self->priv->queue));
	iter = _tmp0_;
	while (TRUE) {
		gboolean _tmp1_;
		gpointer _tmp2_ = NULL;
		TimedQueueElement* e;
		gboolean _tmp3_;
		_tmp1_ = gee_iterator_next (iter);
		if (!_tmp1_) {
			break;
		}
		_tmp2_ = gee_iterator_get (iter);
		e = (TimedQueueElement*) _tmp2_;
		_tmp3_ = self->priv->equal_func (item, e->item);
		if (_tmp3_) {
			gee_iterator_remove (iter);
			result = TRUE;
			_timed_queue_element_unref0 (e);
			_g_object_unref0 (iter);
			return result;
		}
		_timed_queue_element_unref0 (e);
	}
	result = FALSE;
	_g_object_unref0 (iter);
	return result;
}


gboolean timed_queue_remove_first (TimedQueue* self, gconstpointer item) {
	return TIMED_QUEUE_GET_CLASS (self)->remove_first (self, item);
}


static gulong timed_queue_calc_ready_time (TimedQueue* self) {
	gulong result = 0UL;
	gulong _tmp0_;
	g_return_val_if_fail (IS_TIMED_QUEUE (self), 0UL);
	_tmp0_ = now_ms ();
	result = _tmp0_ + ((gulong) self->priv->hold_msec);
	return result;
}


static gboolean timed_queue_on_heartbeat (TimedQueue* self) {
	gboolean result = FALSE;
	gulong now;
	g_return_val_if_fail (IS_TIMED_QUEUE (self), FALSE);
	if (self->priv->paused_state) {
		result = TRUE;
		return result;
	}
	now = (gulong) 0;
	{
		gboolean _tmp0_;
		_tmp0_ = TRUE;
		while (TRUE) {
			gint _tmp1_;
			gpointer _tmp2_ = NULL;
			TimedQueueElement* head;
			gboolean _tmp4_ = FALSE;
			gpointer _tmp5_ = NULL;
			TimedQueueElement* h;
			if (!_tmp0_) {
			}
			_tmp0_ = FALSE;
			_tmp1_ = gee_collection_get_size (GEE_COLLECTION (self->priv->queue));
			if (_tmp1_ == 0) {
				break;
			}
			_tmp2_ = sorted_list_get_at (self->priv->queue, 0);
			head = (TimedQueueElement*) _tmp2_;
			g_assert (head != NULL);
			if (now == 0) {
				gulong _tmp3_;
				_tmp3_ = now_ms ();
				now = _tmp3_;
			}
			if (head->ready > now) {
				_timed_queue_element_unref0 (head);
				break;
			}
			if (self->priv->dequeue_spacing_msec != 0) {
				_tmp4_ = (now - self->priv->last_dequeue) < self->priv->dequeue_spacing_msec;
			} else {
				_tmp4_ = FALSE;
			}
			if (_tmp4_) {
				_timed_queue_element_unref0 (head);
				break;
			}
			_tmp5_ = sorted_list_remove_at (self->priv->queue, 0);
			h = (TimedQueueElement*) _tmp5_;
			g_assert (head == h);
			timed_queue_notify_dequeued (self, head->item);
			self->priv->last_dequeue = now;
			if (self->priv->dequeue_spacing_msec != 0) {
				_timed_queue_element_unref0 (h);
				_timed_queue_element_unref0 (head);
				break;
			}
			_timed_queue_element_unref0 (h);
			_timed_queue_element_unref0 (head);
		}
	}
	result = TRUE;
	return result;
}


gint timed_queue_get_size (TimedQueue* self) {
	return TIMED_QUEUE_GET_CLASS (self)->get_size (self);
}


static gint timed_queue_real_get_size (TimedQueue* base) {
	gint result;
	TimedQueue* self;
	gint _tmp0_;
	self = base;
	_tmp0_ = gee_collection_get_size (GEE_COLLECTION (self->priv->queue));
	result = _tmp0_;
	return result;
}


static void timed_queue_real_paused (TimedQueue* self, gboolean is_paused) {
	g_return_if_fail (IS_TIMED_QUEUE (self));
}


static TimedQueueElement* timed_queue_element_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, gconstpointer item, gulong ready) {
	TimedQueueElement* self = NULL;
	gpointer _tmp0_;
	gconstpointer _tmp1_;
	self = (TimedQueueElement*) g_type_create_instance (object_type);
	self->priv->g_type = g_type;
	self->priv->g_dup_func = g_dup_func;
	self->priv->g_destroy_func = g_destroy_func;
	_tmp0_ = (_tmp1_ = item, ((_tmp1_ == NULL) || (g_dup_func == NULL)) ? ((gpointer) _tmp1_) : g_dup_func ((gpointer) _tmp1_));
	_g_destroy_func0 (self->item);
	self->item = _tmp0_;
	self->ready = ready;
	return self;
}


static TimedQueueElement* timed_queue_element_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, gconstpointer item, gulong ready) {
	return timed_queue_element_construct (TIMED_QUEUE_TYPE_ELEMENT, g_type, g_dup_func, g_destroy_func, item, ready);
}


static gint64 timed_queue_element_comparator (void* a, void* b) {
	gint64 result = 0LL;
	result = ((gint64) TIMED_QUEUE_ELEMENT ((TimedQueueElement*) a)->ready) - ((gint64) TIMED_QUEUE_ELEMENT ((TimedQueueElement*) b)->ready);
	return result;
}


static void timed_queue_value_element_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void timed_queue_value_element_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		timed_queue_element_unref (value->data[0].v_pointer);
	}
}


static void timed_queue_value_element_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = timed_queue_element_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer timed_queue_value_element_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* timed_queue_value_element_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		TimedQueueElement* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = timed_queue_element_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* timed_queue_value_element_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	TimedQueueElement** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags & G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = timed_queue_element_ref (value->data[0].v_pointer);
	}
	return NULL;
}


static GParamSpec* timed_queue_param_spec_element (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	TimedQueueParamSpecElement* spec;
	g_return_val_if_fail (g_type_is_a (object_type, TIMED_QUEUE_TYPE_ELEMENT), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


static gpointer timed_queue_value_get_element (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TIMED_QUEUE_TYPE_ELEMENT), NULL);
	return value->data[0].v_pointer;
}


static void timed_queue_value_set_element (GValue* value, gpointer v_object) {
	TimedQueueElement* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TIMED_QUEUE_TYPE_ELEMENT));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, TIMED_QUEUE_TYPE_ELEMENT));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		timed_queue_element_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		timed_queue_element_unref (old);
	}
}


static void timed_queue_value_take_element (GValue* value, gpointer v_object) {
	TimedQueueElement* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TIMED_QUEUE_TYPE_ELEMENT));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, TIMED_QUEUE_TYPE_ELEMENT));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		timed_queue_element_unref (old);
	}
}


static void timed_queue_element_class_init (TimedQueueElementClass * klass) {
	timed_queue_element_parent_class = g_type_class_peek_parent (klass);
	TIMED_QUEUE_ELEMENT_CLASS (klass)->finalize = timed_queue_element_finalize;
	g_type_class_add_private (klass, sizeof (TimedQueueElementPrivate));
}


static void timed_queue_element_instance_init (TimedQueueElement * self) {
	self->priv = TIMED_QUEUE_ELEMENT_GET_PRIVATE (self);
	self->ref_count = 1;
}


static void timed_queue_element_finalize (TimedQueueElement* obj) {
	TimedQueueElement * self;
	self = TIMED_QUEUE_ELEMENT (obj);
	((self->item == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (self->item = (self->priv->g_destroy_func (self->item), NULL));
}


static GType timed_queue_element_get_type (void) {
	static volatile gsize timed_queue_element_type_id__volatile = 0;
	if (g_once_init_enter (&timed_queue_element_type_id__volatile)) {
		static const GTypeValueTable g_define_type_value_table = { timed_queue_value_element_init, timed_queue_value_element_free_value, timed_queue_value_element_copy_value, timed_queue_value_element_peek_pointer, "p", timed_queue_value_element_collect_value, "p", timed_queue_value_element_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (TimedQueueElementClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) timed_queue_element_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (TimedQueueElement), 0, (GInstanceInitFunc) timed_queue_element_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		GType timed_queue_element_type_id;
		timed_queue_element_type_id = g_type_register_fundamental (g_type_fundamental_next (), "TimedQueueElement", &g_define_type_info, &g_define_type_fundamental_info, 0);
		g_once_init_leave (&timed_queue_element_type_id__volatile, timed_queue_element_type_id);
	}
	return timed_queue_element_type_id__volatile;
}


static gpointer timed_queue_element_ref (gpointer instance) {
	TimedQueueElement* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


static void timed_queue_element_unref (gpointer instance) {
	TimedQueueElement* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		TIMED_QUEUE_ELEMENT_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}


static void value_timed_queue_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void value_timed_queue_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		timed_queue_unref (value->data[0].v_pointer);
	}
}


static void value_timed_queue_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = timed_queue_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer value_timed_queue_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* value_timed_queue_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		TimedQueue* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = timed_queue_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* value_timed_queue_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	TimedQueue** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags & G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = timed_queue_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* param_spec_timed_queue (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	ParamSpecTimedQueue* spec;
	g_return_val_if_fail (g_type_is_a (object_type, TYPE_TIMED_QUEUE), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer value_get_timed_queue (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TYPE_TIMED_QUEUE), NULL);
	return value->data[0].v_pointer;
}


void value_set_timed_queue (GValue* value, gpointer v_object) {
	TimedQueue* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TYPE_TIMED_QUEUE));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, TYPE_TIMED_QUEUE));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		timed_queue_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		timed_queue_unref (old);
	}
}


void value_take_timed_queue (GValue* value, gpointer v_object) {
	TimedQueue* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TYPE_TIMED_QUEUE));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, TYPE_TIMED_QUEUE));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		timed_queue_unref (old);
	}
}


static void timed_queue_class_init (TimedQueueClass * klass) {
	timed_queue_parent_class = g_type_class_peek_parent (klass);
	TIMED_QUEUE_CLASS (klass)->finalize = timed_queue_finalize;
	g_type_class_add_private (klass, sizeof (TimedQueuePrivate));
	TIMED_QUEUE_CLASS (klass)->notify_dequeued = timed_queue_real_notify_dequeued;
	TIMED_QUEUE_CLASS (klass)->clear = timed_queue_real_clear;
	TIMED_QUEUE_CLASS (klass)->contains = timed_queue_real_contains;
	TIMED_QUEUE_CLASS (klass)->enqueue = timed_queue_real_enqueue;
	TIMED_QUEUE_CLASS (klass)->enqueue_many = timed_queue_real_enqueue_many;
	TIMED_QUEUE_CLASS (klass)->remove_first = timed_queue_real_remove_first;
	TIMED_QUEUE_CLASS (klass)->paused = timed_queue_real_paused;
	TIMED_QUEUE_CLASS (klass)->get_size = timed_queue_real_get_size;
	g_signal_new ("paused", TYPE_TIMED_QUEUE, G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (TimedQueueClass, paused), NULL, NULL, g_cclosure_marshal_VOID__BOOLEAN, G_TYPE_NONE, 1, G_TYPE_BOOLEAN);
}


static void timed_queue_instance_init (TimedQueue * self) {
	self->priv = TIMED_QUEUE_GET_PRIVATE (self);
	self->priv->timer_id = (guint) 0;
	self->priv->dequeue_spacing_msec = (guint) 0;
	self->priv->last_dequeue = (gulong) 0;
	self->priv->paused_state = FALSE;
	self->ref_count = 1;
}


static void timed_queue_finalize (TimedQueue* obj) {
	TimedQueue * self;
	self = TIMED_QUEUE (obj);
	if (self->priv->timer_id != 0) {
		g_source_remove (self->priv->timer_id);
	}
	(self->priv->callback_target_destroy_notify == NULL) ? NULL : (self->priv->callback_target_destroy_notify (self->priv->callback_target), NULL);
	self->priv->callback = NULL;
	self->priv->callback_target = NULL;
	self->priv->callback_target_destroy_notify = NULL;
	_g_object_unref0 (self->priv->queue);
}


GType timed_queue_get_type (void) {
	static volatile gsize timed_queue_type_id__volatile = 0;
	if (g_once_init_enter (&timed_queue_type_id__volatile)) {
		static const GTypeValueTable g_define_type_value_table = { value_timed_queue_init, value_timed_queue_free_value, value_timed_queue_copy_value, value_timed_queue_peek_pointer, "p", value_timed_queue_collect_value, "p", value_timed_queue_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (TimedQueueClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) timed_queue_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (TimedQueue), 0, (GInstanceInitFunc) timed_queue_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		GType timed_queue_type_id;
		timed_queue_type_id = g_type_register_fundamental (g_type_fundamental_next (), "TimedQueue", &g_define_type_info, &g_define_type_fundamental_info, 0);
		g_once_init_leave (&timed_queue_type_id__volatile, timed_queue_type_id);
	}
	return timed_queue_type_id__volatile;
}


gpointer timed_queue_ref (gpointer instance) {
	TimedQueue* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void timed_queue_unref (gpointer instance) {
	TimedQueue* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		TIMED_QUEUE_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}


HashTimedQueue* hash_timed_queue_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, guint hold_msec, DequeuedCallback callback, void* callback_target, GHashFunc hash_func, GEqualFunc equal_func, gint priority) {
	HashTimedQueue* self = NULL;
	GeeHashMap* _tmp0_ = NULL;
	self = (HashTimedQueue*) timed_queue_construct (object_type, g_type, (GBoxedCopyFunc) g_dup_func, g_destroy_func, hold_msec, callback, callback_target, equal_func, priority);
	self->priv->g_type = g_type;
	self->priv->g_dup_func = g_dup_func;
	self->priv->g_destroy_func = g_destroy_func;
	_tmp0_ = gee_hash_map_new (g_type, (GBoxedCopyFunc) g_dup_func, g_destroy_func, G_TYPE_INT, NULL, NULL, hash_func, equal_func, NULL);
	_g_object_unref0 (self->priv->item_count);
	self->priv->item_count = _tmp0_;
	return self;
}


HashTimedQueue* hash_timed_queue_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, guint hold_msec, DequeuedCallback callback, void* callback_target, GHashFunc hash_func, GEqualFunc equal_func, gint priority) {
	return hash_timed_queue_construct (TYPE_HASH_TIMED_QUEUE, g_type, g_dup_func, g_destroy_func, hold_msec, callback, callback_target, hash_func, equal_func, priority);
}


static void hash_timed_queue_real_notify_dequeued (TimedQueue* base, gconstpointer item) {
	HashTimedQueue * self;
	self = HASH_TIMED_QUEUE (base);
	hash_timed_queue_removed (self, item);
	TIMED_QUEUE_CLASS (hash_timed_queue_parent_class)->notify_dequeued (TIMED_QUEUE (self), item);
}


static void hash_timed_queue_real_clear (TimedQueue* base) {
	HashTimedQueue * self;
	self = HASH_TIMED_QUEUE (base);
	gee_abstract_map_clear (GEE_ABSTRACT_MAP (self->priv->item_count));
	TIMED_QUEUE_CLASS (hash_timed_queue_parent_class)->clear (TIMED_QUEUE (self));
}


static gboolean hash_timed_queue_real_contains (TimedQueue* base, gconstpointer item) {
	HashTimedQueue * self;
	gboolean result = FALSE;
	gboolean _tmp0_;
	self = HASH_TIMED_QUEUE (base);
	_tmp0_ = gee_abstract_map_has_key (GEE_ABSTRACT_MAP (self->priv->item_count), item);
	result = _tmp0_;
	return result;
}


static gboolean hash_timed_queue_real_enqueue (TimedQueue* base, gconstpointer item) {
	HashTimedQueue * self;
	gboolean result = FALSE;
	gboolean _tmp0_;
	gint _tmp1_ = 0;
	gboolean _tmp2_;
	self = HASH_TIMED_QUEUE (base);
	_tmp0_ = TIMED_QUEUE_CLASS (hash_timed_queue_parent_class)->enqueue (TIMED_QUEUE (self), item);
	if (!_tmp0_) {
		result = FALSE;
		return result;
	}
	_tmp2_ = gee_abstract_map_has_key (GEE_ABSTRACT_MAP (self->priv->item_count), item);
	if (_tmp2_) {
		gpointer _tmp3_ = NULL;
		_tmp3_ = gee_abstract_map_get (GEE_ABSTRACT_MAP (self->priv->item_count), item);
		_tmp1_ = GPOINTER_TO_INT (_tmp3_) + 1;
	} else {
		_tmp1_ = 1;
	}
	gee_abstract_map_set (GEE_ABSTRACT_MAP (self->priv->item_count), item, GINT_TO_POINTER (_tmp1_));
	result = TRUE;
	return result;
}


static gboolean hash_timed_queue_real_enqueue_many (TimedQueue* base, GeeCollection* items) {
	HashTimedQueue * self;
	gboolean result = FALSE;
	gboolean _tmp0_;
	self = HASH_TIMED_QUEUE (base);
	g_return_val_if_fail (GEE_IS_COLLECTION (items), FALSE);
	_tmp0_ = TIMED_QUEUE_CLASS (hash_timed_queue_parent_class)->enqueue_many (TIMED_QUEUE (self), items);
	if (!_tmp0_) {
		result = FALSE;
		return result;
	}
	{
		GeeIterator* _tmp1_ = NULL;
		GeeIterator* _item_it;
		_tmp1_ = gee_iterable_iterator (GEE_ITERABLE (items));
		_item_it = _tmp1_;
		while (TRUE) {
			gboolean _tmp2_;
			gpointer _tmp3_ = NULL;
			gpointer item;
			gint _tmp4_ = 0;
			gboolean _tmp5_;
			_tmp2_ = gee_iterator_next (_item_it);
			if (!_tmp2_) {
				break;
			}
			_tmp3_ = gee_iterator_get (_item_it);
			item = _tmp3_;
			_tmp5_ = gee_abstract_map_has_key (GEE_ABSTRACT_MAP (self->priv->item_count), item);
			if (_tmp5_) {
				gpointer _tmp6_ = NULL;
				_tmp6_ = gee_abstract_map_get (GEE_ABSTRACT_MAP (self->priv->item_count), item);
				_tmp4_ = GPOINTER_TO_INT (_tmp6_) + 1;
			} else {
				_tmp4_ = 1;
			}
			gee_abstract_map_set (GEE_ABSTRACT_MAP (self->priv->item_count), item, GINT_TO_POINTER (_tmp4_));
			((item == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (item = (self->priv->g_destroy_func (item), NULL));
		}
		_g_object_unref0 (_item_it);
	}
	result = TRUE;
	return result;
}


static gboolean hash_timed_queue_real_remove_first (TimedQueue* base, gconstpointer item) {
	HashTimedQueue * self;
	gboolean result = FALSE;
	gboolean _tmp0_;
	self = HASH_TIMED_QUEUE (base);
	_tmp0_ = TIMED_QUEUE_CLASS (hash_timed_queue_parent_class)->remove_first (TIMED_QUEUE (self), item);
	if (!_tmp0_) {
		result = FALSE;
		return result;
	}
	hash_timed_queue_removed (self, item);
	result = TRUE;
	return result;
}


static void hash_timed_queue_removed (HashTimedQueue* self, gconstpointer item) {
	gboolean _tmp0_;
	gpointer _tmp1_ = NULL;
	gint count;
	g_return_if_fail (IS_HASH_TIMED_QUEUE (self));
	_tmp0_ = gee_abstract_map_has_key (GEE_ABSTRACT_MAP (self->priv->item_count), item);
	g_assert (_tmp0_);
	_tmp1_ = gee_abstract_map_get (GEE_ABSTRACT_MAP (self->priv->item_count), item);
	count = GPOINTER_TO_INT (_tmp1_);
	g_assert (count > 0);
	count = count - 1;
	if (count == 0) {
		gee_abstract_map_unset (GEE_ABSTRACT_MAP (self->priv->item_count), item, NULL);
	} else {
		gee_abstract_map_set (GEE_ABSTRACT_MAP (self->priv->item_count), item, GINT_TO_POINTER (count));
	}
}


static void hash_timed_queue_class_init (HashTimedQueueClass * klass) {
	hash_timed_queue_parent_class = g_type_class_peek_parent (klass);
	TIMED_QUEUE_CLASS (klass)->finalize = hash_timed_queue_finalize;
	g_type_class_add_private (klass, sizeof (HashTimedQueuePrivate));
	TIMED_QUEUE_CLASS (klass)->notify_dequeued = hash_timed_queue_real_notify_dequeued;
	TIMED_QUEUE_CLASS (klass)->clear = hash_timed_queue_real_clear;
	TIMED_QUEUE_CLASS (klass)->contains = hash_timed_queue_real_contains;
	TIMED_QUEUE_CLASS (klass)->enqueue = hash_timed_queue_real_enqueue;
	TIMED_QUEUE_CLASS (klass)->enqueue_many = hash_timed_queue_real_enqueue_many;
	TIMED_QUEUE_CLASS (klass)->remove_first = hash_timed_queue_real_remove_first;
}


static void hash_timed_queue_instance_init (HashTimedQueue * self) {
	self->priv = HASH_TIMED_QUEUE_GET_PRIVATE (self);
}


static void hash_timed_queue_finalize (TimedQueue* obj) {
	HashTimedQueue * self;
	self = HASH_TIMED_QUEUE (obj);
	_g_object_unref0 (self->priv->item_count);
	TIMED_QUEUE_CLASS (hash_timed_queue_parent_class)->finalize (obj);
}


GType hash_timed_queue_get_type (void) {
	static volatile gsize hash_timed_queue_type_id__volatile = 0;
	if (g_once_init_enter (&hash_timed_queue_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (HashTimedQueueClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) hash_timed_queue_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (HashTimedQueue), 0, (GInstanceInitFunc) hash_timed_queue_instance_init, NULL };
		GType hash_timed_queue_type_id;
		hash_timed_queue_type_id = g_type_register_static (TYPE_TIMED_QUEUE, "HashTimedQueue", &g_define_type_info, 0);
		g_once_init_leave (&hash_timed_queue_type_id__volatile, hash_timed_queue_type_id);
	}
	return hash_timed_queue_type_id__volatile;
}



